@@ -15,9 +15,11 @@ from sqlvalidator.grammar.sql import (
     Column,
     CombinedQueries,
     Condition,
+    CountFunctionCall,
     DatePartExtraction,
     ExceptClause,
     Expression,
+    FilteredFunctionCall,
     Float,
     FunctionCall,
     GroupByClause,
@@ -41,6 +43,8 @@ from sqlvalidator.grammar.sql import (
     UsingClause,
     WhereClause,
     WindowFrameClause,
+    WithQuery,
+    WithStatement,
 )
 from sqlvalidator.grammar.tokeniser import (
     get_tokens_until_closing_parenthesis,
@@ -60,6 +64,8 @@ class SQLStatementParser:
         next_token = next(tokens)
         if lower(next_token) == "select":
             return SelectStatementParser.parse(tokens)
+        if lower(next_token) == "with":
+            return WithStatementParser.parse(tokens)
         raise ParsingError
 
 
@@ -111,12 +117,17 @@ class SelectStatementParser:
 
         if lower(next_token) == "group":
             next_token = next(tokens, None)
+            group_each_by = False
+            if lower(next_token) == "each":
+                group_each_by = True
+                next_token = next(tokens, None)
             if not lower(next_token) == "by":
                 raise ParsingError("Missing BY after GROUP")
             expression_tokens, next_token = get_tokens_until_one_of(
                 tokens, ["having", "order", "limit", "offset", ";"]
             )
             group_by_clause = GroupByParser.parse(iter(expression_tokens))
+            group_by_clause.group_each_by = group_each_by
         else:
             group_by_clause = None
 
@@ -179,7 +190,7 @@ class FromStatementParser:
                 argument = FromStatementParser.parse(iter(argument_tokens))
             expression = Parenthesis(argument)
             next_token = next(tokens, None)
-        elif next_token == "'" or next_token == '"' or next_token == "`":
+        elif next_token in String.QUOTES:
             table_name = StringParser.parse(tokens, next_token)
             next_token = next(tokens, None)
             while next_token == ".":
@@ -199,8 +210,7 @@ class FromStatementParser:
             next_token = next(tokens, None)
         else:
             if lower(next_token) == "unnest":
-                expression = UnnestParser.parse(tokens)
-                next_token = next(tokens, None)
+                expression, next_token = UnnestParser.parse(tokens)
             else:
                 argument_tokens, next_token = get_tokens_until_one_of(
                     tokens,
@@ -220,6 +230,8 @@ class FromStatementParser:
             else:
                 with_as = False
                 alias = next_token
+            if alias in String.QUOTES:
+                alias = StringParser.parse(tokens, alias)
             expression = Alias(expression, alias, with_as)
             next_token = next(tokens, None)
 
@@ -250,6 +262,8 @@ class FromStatementParser:
                 else:
                     with_as = False
                     alias = next_token
+                if alias in String.QUOTES:
+                    alias = StringParser.parse(tokens, alias)
                 expression = Alias(expression, alias, with_as)
                 next_token = next(tokens, None)
 
@@ -280,8 +294,6 @@ class FromStatementParser:
             elif lower(on_or_using) == "using":
                 expressions, _ = ExpressionParser.parse(iter(expression_tokens))
                 using = UsingClause(expressions)
-            elif join_type not in ("CROSS JOIN", ","):
-                raise ParsingError("Missing ON or USING for join")
 
             expression = Join(join_type, left_expr, right_expr, on=on, using=using)
 
@@ -292,6 +304,8 @@ class FromStatementParser:
                 else:
                     with_as = False
                     alias = next_token
+                if alias in String.QUOTES:
+                    alias = StringParser.parse(tokens, alias)
                 expression = Alias(expression, alias, with_as)
                 next_token = next(tokens, None)
 
@@ -327,6 +341,8 @@ class UnnestParser:
             else:
                 with_as = False
                 alias = next_token
+            if alias in String.QUOTES:
+                alias = StringParser.parse(tokens, alias)
             expression = Alias(expression, alias, with_as)
             next_token = next(tokens, None)
 
@@ -350,15 +366,42 @@ class UnnestParser:
                     offset_alias = next(tokens)
                 else:
                     offset_alias = next_token
+                next_token = next(tokens, None)
 
-        return Unnest(
-            expression,
-            with_offset=with_offset,
-            with_offset_as=with_offset_as,
-            offset_alias=offset_alias,
+        return (
+            Unnest(
+                expression,
+                with_offset=with_offset,
+                with_offset_as=with_offset_as,
+                offset_alias=offset_alias,
+            ),
+            next_token,
         )
 
 
+class WithStatementParser:
+    @staticmethod
+    def parse(tokens):
+        with_queries = []
+
+        with_query_name = None
+        while lower(with_query_name) != "select":
+            with_query_name = next(tokens)
+            next_token = next(tokens)
+            assert lower(next_token) == "as", next_token
+            next_token = next(tokens)
+            assert lower(next_token) == "(", next_token
+            with_statement = SQLStatementParser.parse(
+                iter(get_tokens_until_closing_parenthesis(tokens))
+            )
+
+            with_queries.append(WithQuery(with_query_name, with_statement))
+            with_query_name = next(tokens)
+
+        select_statement = SelectStatementParser.parse(tokens)
+        return WithStatement(with_queries, select_statement)
+
+
 class SetOperatorTypeParser:
     @staticmethod
     def parse(tokens):
@@ -625,6 +668,20 @@ class ExpressionParser:
                         order_bys=order_bys,
                         limit=limit,
                     )
+                elif lower(main_token) == "count":
+                    next_token = next(tokens)
+                    if lower(next_token) == "distinct":
+                        distinct = True
+                        first_token = None
+                    else:
+                        distinct = False
+                        first_token = next_token
+
+                    argument_tokens = get_tokens_until_closing_parenthesis(
+                        tokens, first_token=first_token
+                    )
+                    arguments = ExpressionListParser.parse(iter(argument_tokens))
+                    expression = CountFunctionCall(*arguments, distinct=distinct)
                 else:
                     argument_tokens = get_tokens_until_closing_parenthesis(tokens)
                     arguments_can_be_type = can_be_type or any(
@@ -634,7 +691,21 @@ class ExpressionParser:
                         iter(argument_tokens), can_be_type=arguments_can_be_type
                     )
                     expression = FunctionCall(main_token, *arguments)
+
                 next_token = next(tokens, None)
+                if next_token and lower(next_token) == "filter":
+                    next_next_token = next(tokens)
+                    assert next_next_token == "(", next_next_token
+                    argument_tokens = get_tokens_until_closing_parenthesis(tokens)
+                    assert lower(argument_tokens[0]) == "where", argument_tokens
+                    filter_condition, next_token = ExpressionParser.parse(
+                        iter(argument_tokens[1:]),
+                        can_alias=False,
+                    )
+
+                    expression = FilteredFunctionCall(expression, filter_condition)
+                    next_token = next(tokens, None)
+
             elif (
                 next_token is not None
                 and lower(main_token) in DatePartExtraction.PARTS
@@ -735,22 +806,28 @@ class ExpressionParser:
                 tokens, is_right_hand=True, until_one_of=until_one_of
             )
             expression = ChainedColumns(expression, right_hand)
+            if next_token is not None and next_token == "[":
+                argument_tokens, next_token = get_tokens_until_one_of(
+                    tokens, stop_words=["]"]
+                )
+                arguments = ExpressionListParser.parse(iter(argument_tokens))
+                expression = Index(expression, arguments)
+                next_token = next(tokens, None)
 
         if is_right_hand:
             return expression, next_token
 
         if lower(next_token) in Condition.PREDICATES:
+            first_token = None
             symbol = next_token
             if lower(next_token) == "is":
                 next_next_token = next(tokens)
-                first_token = None
                 if lower(next_next_token) == "not":
                     symbol = "is not"
                 else:
                     first_token = next_next_token
             elif lower(next_token) == "not":
                 next_next_token = next(tokens)
-                first_token = None
                 if lower(next_next_token) == "in":
                     symbol = "not in"
                 else:
@@ -792,7 +869,14 @@ class ExpressionParser:
             right_hand, next_token = ExpressionParser.parse(
                 tokens, until_one_of=until_one_of
             )
+            right_alias = None
+            if isinstance(right_hand, Alias):
+                right_alias = right_hand
+                right_hand = right_hand.expression
             expression = BooleanCondition(symbol, left_hand, right_hand)
+            if right_alias is not None:
+                right_alias.expression = expression
+                expression = right_alias
 
         if lower(next_token) == "except":
             opening_parenthesis = next(tokens, None)
@@ -806,9 +890,7 @@ class ExpressionParser:
         if (
             next_token is not None
             and next_token != ")"
-            and next_token != "'"
-            and next_token != '"'
-            and next_token != "`"
+            and not (next_token in String.QUOTES and isinstance(expression, String))
             and next_token != ";"
             and lower(next_token) not in until_one_of
             and can_alias
@@ -821,6 +903,8 @@ class ExpressionParser:
             else:
                 with_as = False
                 alias = next_token
+            if alias in String.QUOTES:
+                alias = StringParser.parse(tokens, alias)
             return Alias(expression, alias, with_as), next(tokens, None)
         return expression, next_token
 
