@@ -1,4 +1,4 @@
-from typing import Any
+from typing import Any, List, Optional, Set
 
 from sqlvalidator.grammar.tokeniser import lower
 
@@ -90,7 +90,10 @@ class SelectStatement:
             statement_str += "\nWHERE{}".format(transform(self.where_clause))
 
         if self.group_by_clause:
-            statement_str += "\nGROUP BY{}".format(transform(self.group_by_clause))
+            statement_str += "\nGROUP {}BY{}".format(
+                "EACH " if self.group_by_clause.group_each_by else "",
+                transform(self.group_by_clause),
+            )
 
         if self.having_clause:
             statement_str += "\nHAVING{}".format(transform(self.having_clause))
@@ -110,16 +113,16 @@ class SelectStatement:
             statement_str += ";"
         return statement_str
 
-    def validate(self):
+    def validate(self, known_fields: Optional[Set[str]] = None) -> list:
         errors = []
+        known_fields = known_fields or set()
+
         if isinstance(self.from_statement, Parenthesis) and isinstance(
             self.from_statement.args[0], SelectStatement
         ):
-            known_fields = self.from_statement.args[0].known_fields
+            known_fields = known_fields | self.from_statement.args[0].known_fields
         elif isinstance(self.from_statement, Table):
-            known_fields = {"*"}
-        else:
-            known_fields = set()
+            known_fields = known_fields | {"*"}
 
         for e in self.expressions:
             errors += e.validate(known_fields)
@@ -138,13 +141,16 @@ class SelectStatement:
         return errors
 
     @property
-    def known_fields(self):
-        fields = []
+    def known_fields(self) -> Set[str]:
+        fields = set()
         for e in self.expressions:
             if isinstance(e, Column):
-                fields.append(e.value)
+                fields.add(e.value)
             elif isinstance(e, Alias):
-                fields.append(e.alias)
+                if isinstance(e.alias, Column):
+                    fields.add(e.alias.value)
+                else:
+                    fields.add(e.alias)
         return fields
 
     def __eq__(self, other):
@@ -219,7 +225,7 @@ class Expression:
     def __eq__(self, other):
         return type(self) == type(other) and self.value == other.value
 
-    def validate(self, known_fields):
+    def validate(self, known_fields: Set[str]) -> list:
         return []
 
     @property
@@ -236,7 +242,7 @@ class WhereClause(Expression):
             return "\n " + transformed_value.replace("\n", "\n ")
         return " " + transformed_value
 
-    def validate(self, known_fields):
+    def validate(self, known_fields: Set[str]) -> list:
         errors = super().validate(known_fields)
         errors += self.value.validate(known_fields)
         if self.value.return_type != bool:
@@ -255,6 +261,7 @@ class GroupByClause(Expression):
     def __init__(self, *args, rollup=False):
         self.args = args
         self.rollup = rollup
+        self.group_each_by = False
 
     def __str__(self):
         if len(self.args) > 1:
@@ -278,6 +285,7 @@ class GroupByClause(Expression):
             and len(self.args) == len(other.args)
             and all(a == o for a, o in zip(self.args, other.args))
             and self.rollup == other.rollup
+            and self.group_each_by == other.group_each_by
         )
 
     def validate(self, known_fields, select_expressions):
@@ -293,7 +301,11 @@ class GroupByClause(Expression):
                 )
             elif (
                 isinstance(arg, (Column, String))
-                and arg.value not in known_fields
+                and (
+                    arg.value not in known_fields
+                    and arg.value
+                    not in [e.alias for e in select_expressions if isinstance(e, Alias)]
+                )
                 and "*" not in known_fields
             ):
                 errors.append('column "{}" does not exist'.format(arg.value))
@@ -421,11 +433,11 @@ class LimitClause(Expression):
         value = self.value
         while isinstance(value, Parenthesis):
             value = value.value
-        if self.value.return_type != int:
-            errors.append("argument of OFFSET must not contain variables")
+        if value.return_type != int or not isinstance(value, Integer):
+            errors.append("argument of LIMIT must not contain variables")
         else:
             if isinstance(value, Integer) and value.value < 0:
-                errors.append("OFFSET must not be negative")
+                errors.append("LIMIT must not be negative")
         return errors
 
 
@@ -435,14 +447,54 @@ class OffsetClause(Expression):
         value = self.value
         while isinstance(value, Parenthesis):
             value = value.value
-        if self.value.return_type != int:
-            errors.append("argument of LIMIT must be integer")
+        if value.return_type != int or not isinstance(value, Integer):
+            errors.append("argument of OFFSET must be integer")
         else:
             if isinstance(value, Integer) and value.value < 0:
-                errors.append("LIMIT must not be negative")
+                errors.append("OFFSET must not be negative")
         return errors
 
 
+class WithQuery(Expression):
+    def __init__(self, name: str, statement: SelectStatement):
+        self.name = name
+        # todo: column name, for recursivity
+        self.statement = statement
+
+    def __str__(self):
+        return "{} AS (\n{}\n)".format(
+            self.name, self.statement.transform(is_subquery=True)
+        )
+
+    def __eq__(self, other):
+        return (
+            type(self) == type(other)
+            and self.name == other.name
+            and self.statement == other.statement
+        )
+
+
+class WithStatement(Expression):
+    def __init__(self, with_queries: List[WithQuery], select_statement):
+        # todo: recursive
+        self.with_queries = with_queries
+        self.select_statement = select_statement
+
+    def transform(self):
+        return "WITH {}\n{}".format(
+            ",\n".join(map(transform, self.with_queries)),
+            transform(self.select_statement),
+        )
+
+    def __eq__(self, other):
+        return (
+            type(self) == type(other)
+            and len(self.with_queries) == len(other.with_queries)
+            and all(a == o for a, o in zip(self.with_queries, other.with_queries))
+            and self.select_statement == other.select_statement
+        )
+
+
 class FunctionCall(Expression):
     def __init__(self, function_name, *args):
         self.function_name = function_name
@@ -492,6 +544,22 @@ class CastFunctionCall(FunctionCall):
         )
 
 
+class CountFunctionCall(FunctionCall):
+    def __init__(self, *args, distinct=False):
+        super().__init__("count", args)
+        self.distinct = distinct
+
+    def __str__(self):
+        return "{}({}{})".format(
+            self.function_name.upper(),
+            "DISTINCT " if self.distinct else "",
+            ", ".join(map(transform, *self.args)),
+        )
+
+    def __eq__(self, other):
+        return super().__eq__(other) and self.distinct == other.distinct
+
+
 class ArrayAggFunctionCall(FunctionCall):
     def __init__(
         self,
@@ -515,15 +583,15 @@ class ArrayAggFunctionCall(FunctionCall):
         if self.distinct:
             array_agg_str += "DISTINCT "
 
-        array_agg_str += transform(self.args[0]) + " "
+        array_agg_str += transform(self.args[0])
 
         if self.ignore_nulls:
-            array_agg_str += "IGNORE NULLS "
+            array_agg_str += " IGNORE NULLS"
         elif self.respect_nulls:
-            array_agg_str += "RESPECT NULLS "
+            array_agg_str += " RESPECT NULLS"
 
         if self.order_bys:
-            array_agg_str += "ORDER BY{}".format(
+            array_agg_str += " ORDER BY{}".format(
                 self.order_bys.transform(allow_linebreak=False)
             )
 
@@ -533,6 +601,24 @@ class ArrayAggFunctionCall(FunctionCall):
         return array_agg_str + ")"
 
 
+class FilteredFunctionCall(Expression):
+    def __init__(self, function_call: FunctionCall, filter_condition):
+        self.function_call = function_call
+        self.filter_condition = filter_condition
+
+    def __str__(self):
+        return "{} FILTER (WHERE {})".format(
+            transform(self.function_call), transform(self.filter_condition)
+        )
+
+    def __eq__(self, other):
+        return (
+            type(self) == type(other)
+            and self.function_call == other.function_call
+            and self.filter_condition == other.filter_condition
+        )
+
+
 class AnalyticsClause(Expression):
     def __init__(self, function, partition_by, order_by, frame_clause):
         self.function = function
@@ -666,6 +752,10 @@ class ChainedColumns(Expression):
             and all(a == o for a, o in zip(self.columns, other.columns))
         )
 
+    @property
+    def return_type(self):
+        return self.columns[0].return_type
+
 
 class Type(Expression):
     VALUES = ("int", "float", "day", "month", "timestamp", "int64", "string", "date")
@@ -699,6 +789,7 @@ class DatePartExtraction(Expression):
         "dayofmonth",
         "dayofweek",
         "dayofyear",
+        "date",
     )
 
     def __init__(self, part, date_expression):
@@ -868,7 +959,9 @@ class Alias(Expression):
     def transform(self, expression=None):
         expression = expression or self.expression
         return "{}{}{}".format(
-            expression, " AS " if self.with_as else " ", transform(self.alias)
+            transform(expression),
+            " AS " if self.with_as else " ",
+            transform(self.alias),
         )
 
     def __repr__(self):
@@ -889,6 +982,10 @@ class Alias(Expression):
             errors += self.expression.validate(known_fields)
         return errors
 
+    @property
+    def return_type(self):
+        return self.expression.return_type
+
 
 class Index(Expression):
     def __init__(self, expression, indices):
@@ -930,6 +1027,8 @@ class ArithmaticOperator(Expression):
 
     @property
     def return_type(self):
+        if any(isinstance(a, float) for a in self.args):
+            return float
         return int
 
 
@@ -998,7 +1097,7 @@ class Unnest(Expression):
 
 
 class Join(Expression):
-    VALUES = ("join", "inner", "left", "right", "full", "cross", "outer", ",")
+    VALUES = ("join", "inner", "left", "right", "full", "cross", "outer", ",", "each")
 
     def __init__(self, join_type, left_from, right_from, on, using):
         self.join_type = join_type
@@ -1113,6 +1212,12 @@ class Join(Expression):
             self.using,
         )
 
+    def validate(self, known_fields: Set[str]) -> list:
+        errors = super().validate(known_fields)
+        if self.join_type not in ("CROSS JOIN", ",") and not (self.using or self.on):
+            errors.append("Missing ON or USING for join")
+        return errors
+
 
 class CombinedQueries(Expression):
     SET_OPERATORS = ("union", "intersect", "except", "all")
@@ -1180,6 +1285,7 @@ class Condition(Expression):
         "<",
         "<=",
         ">=",
+        "!=",
         "is",
         "like",
         "<>",
@@ -1270,7 +1376,7 @@ class BooleanCondition(Expression):
         errors = super().validate(known_fields)
         for a in self.args:
             errors += a.validate(known_fields)
-            if a.return_type != bool:
+            if a.return_type != bool and a.return_type != Any:
                 errors.append(
                     "The argument of {} must be type boolean, not type {}".format(
                         self.type.upper(), a.return_type
@@ -1296,6 +1402,10 @@ class Negation(Expression):
             negation += " "
         return negation + transform(self.value)
 
+    @property
+    def return_type(self):
+        return bool
+
 
 class ExceptClause(Expression):
     def __init__(self, expression, args):
