@@ -18,16 +18,16 @@ KEPT_SPLIT_TOKENS = (
     ">",
     ".",
 )
-MERGE_TOKENS = ("<>", "<=", ">=", "<<", ">>", "||")
+MERGE_TOKENS = ("<>", "<=", ">=", "<<", ">>", "||", "!=")
 
 
 def lower(s: Optional[str]) -> Optional[str]:
     return s.lower() if s else s
 
 
-def get_tokens_until_closing_parenthesis(tokens):
+def get_tokens_until_closing_parenthesis(tokens, first_token=None):
     argument_tokens = []
-    next_token = next(tokens, None)
+    next_token = first_token or next(tokens, None)
     count_parenthesis = 0
     while next_token is not None and not (next_token == ")" and count_parenthesis == 0):
         argument_tokens.append(next_token)
@@ -81,7 +81,7 @@ def get_tokens_until_not_in(tokens, kept_words, first_token=None):
 
 
 def split_with_sep(s: str, sep: str):
-    splitted = s.split(sep)
+    splitted = split_with_escaping(s, sep)
     for word in splitted[:-1]:
         if word:
             yield word
@@ -90,6 +90,23 @@ def split_with_sep(s: str, sep: str):
         yield splitted[-1]
 
 
+def split_with_escaping(s: str, sep: str):
+    splitted = []
+    split_iterator = iter(s.split(sep))
+    value = next(split_iterator, None)
+    while value is not None:
+        # Glue values together if defined
+        if value.endswith("\\"):
+            value += sep
+            next_value = next(split_iterator, None)
+            if next_value:
+                value += next_value
+        # Add to list, and keep looping
+        splitted.append(value)
+        value = next(split_iterator, None)
+    return splitted
+
+
 def merge_stream(s, goals):
     for element in s:
         matching_goals = [g for g in goals if g.startswith(element)]
